function score_segments(DHSCutCountsFile, ...
                        mappableFile, ...
                        segmentFile, ...
                        localBgrWidth, ...
                        flankSize, ...
                        transformType, ...
                        isOneSideBkg, ...
                        scoringType, ...
                        outPath, ...
                        outPrefix)

% localBgrWidth should be odd!
if flankSize < floor(localBgrWidth/2)
  error('flank-size must be larger than half of local-background-width!');
end

if outPath(end) ~= '/'
  outPath = [outPath '/'];
end

fpCC = fopen(DHSCutCountsFile, 'r');
fpMap = fopen(mappableFile, 'r');
fpSeg = fopen(segmentFile, 'r');
fpScore = fopen([outPath outPrefix '.scores'], 'w');

% reset seed
%rand('seed', now());
rand('seed', 99999999);
rowCount = 0;


doLast = false;
segLine = fgetl(fpSeg);
tmp = str2num(segLine);
seqId = tmp(1);
segStart = tmp(2);
segStop = tmp(3);

while not(feof(fpSeg)) || doLast
  % read in segments of the same sequence ID
  winStarts = [];
  winStops = [];
  segCount = 1;
  
  winStarts(segCount) = segStart + flankSize;
  winStops(segCount) = segStop + flankSize;
  currSeqId = seqId;
 
  if doLast
    doLast = false;
  end

  while not(feof(fpSeg))
    segLine = fgetl(fpSeg);
    tmp = str2num(segLine);
    seqId = tmp(1);
    segStart = tmp(2);
    segStop = tmp(3);
  
    if seqId == currSeqId
      segCount = segCount + 1;
      winStarts(segCount) = segStart + flankSize;
      winStops(segCount) = segStop + flankSize;
    else
      break;
    end

    % if the last segment has its own sequence ID,
    % need to iterate for the last time
    if feof(fpSeg) && not(seqId == currSeqId)
      doLast = true;
    end
  end


  % read in cut-counts and mappable for current sequence ID
  while rowCount < currSeqId
    rowCount = rowCount + 1;
    ccLine = fgetl(fpCC);
    mapLine = fgetl(fpMap);
  end

  DHSCC = str2num(ccLine);
  DHSCC(isnan(DHSCC)) = 0;

  mappable = str2num(mapLine);
  % ONLY positions mappable in both strands are flagged with '2'
  ix = find(mappable ~= 2);
  % unmappable positions get flag 0
  mappable(ix) = 0;
  % make sure unmappable positions have 0 cuts
  DHSCC(ix) = 0;
  % mappable (both strand) positions get flag 1
  mappable(mappable == 2)= 1; 


  % now compute pvalues for each kmer
  % using local background
  detect_footprints(rowCount, DHSCC, mappable, ...
                    winStarts, winStops, ...
                    transformType, localBgrWidth, flankSize, ...
                    isOneSideBkg, scoringType, fpScore);
end

fclose(fpMap);
fclose(fpCC);
fclose(fpSeg);
fclose(fpScore);




% function
% ??Notice: realStart and realStop take 1~length(DHSCC)
function [inputData] = generate_input(DHSCC, ...
                                      mappable, ...
                                      realStart, ...
                                      realStop, ...
                                      transformType)

cnum = length(DHSCC);
% converting cut-counts to ranks
if transformType == 1
  % set rank = 0 for unmappables
  cutRanks = zeros(1, cnum);
  rk1 = [];
  rk2 = [];
 
  % mask out unmappables for the truncated DHS
  validIx = find(mappable(realStart:realStop) ~= 0) + realStart - 1;   
  if length(validIx) > 0
    validItem = DHSCC(validIx) ;
    [junk, ix1] = sort(validItem);
    rk1(ix1) = 1:length(ix1);
    % rank, assigning by averge rank to ties
    [junk, ix2] = sort(validItem(end:-1:1));
    rk2(ix2) = 1:length(ix2);
    cutRanks(validIx) = (rk1 + rk2(end:-1:1))/2;
  end
  inputData = cutRanks;    

% truncate top 2%
elseif transformType == 2
  inputData = zeros(1, cnum);;
  validIx = find(mappable(realStart:realStop) ~= 0) + realStart -1;  
  if length(validIx) > 0
    validItem = DHSCC(validIx);
    currPrt = prctile(validItem, 95);
    validItem(find(validItem > currPrt)) = floor(currPrt);
    inputData(validIx) = validItem;
  end

% log(.+1) 
elseif transformType == 3
  inputData = ceil(log(DHSCC+1));
end




% function
function detect_footprints(rowNumber, data, mappable, ...
                           winStarts, winStops, ...
                           transformType, localBkWidth, flankSize, ...
                           isOneSideBkg, scoringType, fpScore)

% compute pvalues for each window
[kmAllCount, allPV, allStart, allStop] = compute_scores_local(...
                             rowNumber, data, mappable, ...
                             winStarts, winStops, ...
                             transformType, localBkWidth, flankSize,  ...
                             isOneSideBkg, scoringType);

% output to file
for i = 1:kmAllCount-1
  fprintf(fpScore, '%d\t%d\t%d\t%g\n', ...
          rowNumber, allStart(i), allStop(i), allPV(i));
end






% function
function [kmAllCount, allPV, allStart, allStop] = ...
    compute_scores_local(rowNumber, data, mappable, ...
                         winStarts, winStops, ...
                         transformType, localBkWidth, flankSize, ...
                         isOneSideBkg, scoringType)  

% bkLen is required to be odd.
bkLen = localBkWidth;
bkHalf = floor(bkLen / 2);

winNum = length(winStarts);
allPV = repmat(-1, 1, winNum);
allStart = repmat(-1, 1, winNum);
allStop = repmat(-1, 1, winNum);
kmAllCount = 1;

for i = 1:winNum
  wStart = winStarts(i);
  wStop = winStops(i);

  % define current local background region
  % !! can be revised to add fixed length of backgrd around the foregrd
  c = wStart + floor((wStop-wStart)/2);
  currBkStart = c - bkHalf;
  currBkStop = c + bkHalf;
   
  % generate input of current local background region
  currBkMappable = mappable(currBkStart:currBkStop);

  % transformation may be applied
  if transformType == 0
    currBk = data(currBkStart:currBkStop);
  else
    currBk = generate_input(data(currBkStart:currBkStop), ...
                            currBkMappable,...
                            1, bkLen, transformType);
  end

  if ~isOneSideBkg
    bkRealW = sum(currBkMappable);
    % in currBk, unmappable positions get value 0 
    bkRealCC = floor(sum(currBk));
  end

  % relative coordinates in currBk and currBkMappable
  localStart = wStart - currBkStart + 1;
  localStop = wStop - currBkStart + 1;

  % mask out unmappables from curr window
  realK = sum(currBkMappable(localStart:localStop));

  % !! ADD CRETERIA TO FILTER UNREASONABLE SEGMENTS
  % E.G. MIN MAPPABLE POSITIONS
  %if realK >= minK

    % in currBk, unmappable positions get value 0 
    kmerCC = floor(sum(currBk(localStart:localStop)));

    % two-sided background
    if ~isOneSideBkg
      % binomial pv scoring
      if scoringType == 1
        % compute binomial pvalue of the cut-counts of a kmer, 
        % given the cut-counts of the DHS containing the kmer
        p = realK/bkRealW;
        kmerPV = compute_bino_pv(p, kmerCC, bkRealCC);
        allPV(kmAllCount) = kmerPV;

      % SNR scoring
      elseif scoringType == 2
        kmerSc = compute_snr(realK, kmerCC, bkRealW, bkRealCC);
        allPV(kmAllCount) = kmerSc;
      end

    % one-sided background
    else
      bkLeftW = sum(currBkMappable(1:localStop));
      bkLeftCC = floor(sum(currBk(1:localStop)));

      bkRightW = sum(currBkMappable(localStart:end));
      bkRightCC = floor(sum(currBk(localStart:end)));

      % binomial pv scoring
      if scoringType == 1
        % left side background  
        p = realK/bkLeftW;
        kmerLeftPV = compute_bino_pv(p, kmerCC, bkLeftCC);
        % right side background
        p = realK/bkRightW;
        kmerRightPV = compute_bino_pv(p, kmerCC, bkRightCC);
        % take the max pv
        allPV(kmAllCount) = max(kmerLeftPV, kmerRightPV);

      % SNR scoring
      elseif scoringType == 2
        kmerLeftSc = compute_snr(realK, kmerCC, bkLeftW, bkLeftCC);
        kmerRightSc = compute_snr(realK, kmerCC, bkRightW, bkRightCC);
        allPV(kmAllCount) = max(kmerLeftSc, kmerRightSc);
      end
    end

    allStart(kmAllCount) = wStart;
    allStop(kmAllCount) = wStop;
    kmAllCount = kmAllCount + 1;
  %end
end




% function
function [kmerPV] = compute_bino_pv(p, kmerCC, bkgCC)            

% use normal distribution to approximate binomial
if (bkgCC*p > 10) && (bkgCC*(1-p) > 10)
  u = bkgCC * p;
  sigm = sqrt(bkgCC * p * (1-p));
  kmerPV = normcdf(kmerCC, u, sigm);
else
  kmerPV = binocdf(kmerCC, bkgCC, p);
end

if kmerPV > 0
  kmerPV = log(kmerPV);
else
  kmerPV = bino_logcdf(kmerCC, bkgCC, p);
end




% function
function [kmerSc] = compute_snr(K, kmerCC, bkgW, bkgCC)

% pseudo count to avoid dividing by 0
pseudoC = 0.0001;

% signal: foreground average
% K+1 to avoid K = 0
% K > 0 is guaranteed by contions in previous code, but NOT flkW (below).
signalVal = (kmerCC + pseudoC * (K+1)) / (K + pseudoC * (K+1));

% noise: exclude foreground from the background
flkW = bkgW - K;
flkCC = bkgCC - kmerCC;
noiseVal = (flkCC + pseudoC * (flkW+1)) / (flkW + pseudoC * (flkW+1));

kmerSc =  signalVal / noiseVal;
